import { kebabCase } from "scule";
import { visit } from "unist-util-visit";
import { parseFrontMatter } from "./frontmatter.mjs";
import toMarkdown from "./to-markdown.mjs";
import fromMarkdown from "./from-markdown.mjs";
import syntax from "./micromark-extension/index.mjs";
const toFrontMatter = (yamlString) => `---
${yamlString}
---`;
export default (function({ components = [] } = {}) {
  const data = this.data();
  add("micromarkExtensions", syntax());
  add("fromMarkdownExtensions", fromMarkdown);
  add("toMarkdownExtensions", toMarkdown);
  function add(field, value) {
    if (!data[field]) {
      data[field] = [];
    }
    ;
    data[field].push(value);
  }
  if (components.length) {
    return async (tree, { data: data2 }) => {
      const jobs = [];
      visit(tree, ["textComponent", "leafComponent", "containerComponent"], (node) => {
        bindNode(node, data2);
        const { instance: handler, options } = components.find((c) => c.name === node.name) || {};
        if (handler) {
          jobs.push(handler(options)(node, data2));
        }
      });
      await Promise.all(jobs);
      return tree;
    };
  }
  return (tree, { data: data2 }) => {
    visit(tree, ["textComponent", "leafComponent", "containerComponent"], (node) => {
      bindNode(node, data2);
    });
  };
});
function bindNode(node, data) {
  const nodeData = node.data || (node.data = {});
  node.fmAttributes = getNodeData(node);
  nodeData.hName = kebabCase(node.name);
  nodeData.hProperties = bindData({
    ...node.attributes,
    ...node.fmAttributes
  }, data);
}
function getNodeData(node) {
  if (!node.rawData) {
    return {};
  }
  const yaml = node.rawData;
  const { data } = parseFrontMatter(toFrontMatter(yaml));
  return data;
}
function bindData(data, pageData) {
  const entries = Object.entries(data).map(([key, value]) => {
    if (key.startsWith(":")) {
      return [key, value];
    }
    if (typeof value === "string") {
      return [pageData[value] ? `:${key}` : key, value];
    }
    return [`:${key}`, JSON.stringify(value)];
  });
  return Object.fromEntries(entries);
}
