import { factorySpace } from "micromark-factory-space";
import { markdownLineEnding, asciiAlpha, markdownSpace } from "micromark-util-character";
import { sizeChunks } from "./utils.mjs";
import { Codes, ContainerSequenceSize, SectionSequenceSize } from "./constants.mjs";
import createName from "./factory-name.mjs";
import createLabel from "./factory-label.mjs";
import createAttributes from "./factory-attributes.mjs";
const label = { tokenize: tokenizeLabel, partial: true };
const attributes = { tokenize: tokenizeAttributes, partial: true };
const sectionSeparatorCode = Codes.hash;
const sectionSeparatorLength = 1;
function linePrefixSize(events) {
  let size = 0;
  let index = events.length - 1;
  let tail = events[index];
  while (index >= 0 && tail && tail[1].type === "linePrefix" && tail[0] === "exit") {
    size += sizeChunks(tail[2].sliceStream(tail[1]));
    index -= 1;
    tail = events[index];
  }
  return size;
}
var MarkDownDataSectionState;
(function(MarkDownDataSectionState2) {
  MarkDownDataSectionState2[MarkDownDataSectionState2["NotSeen"] = 0] = "NotSeen";
  MarkDownDataSectionState2[MarkDownDataSectionState2["Open"] = 1] = "Open";
  MarkDownDataSectionState2[MarkDownDataSectionState2["Closed"] = 2] = "Closed";
})(MarkDownDataSectionState || (MarkDownDataSectionState = {}));
function tokenize(effects, ok, nok) {
  const self = this;
  const initialPrefix = linePrefixSize(this.events);
  let sizeOpen = 0;
  let previous;
  const containerSequenceSize = [];
  const data = tokenizeData.call(this, effects, lineStart);
  return start;
  function start(code) {
    if (code !== Codes.colon) {
      throw new Error("expected `:`");
    }
    effects.enter("componentContainer");
    effects.enter("componentContainerFence");
    effects.enter("componentContainerSequence");
    return sequenceOpen(code);
  }
  function tokenizeSectionClosing(effects2, ok2, nok2) {
    let size = 0;
    let sectionIndentSize = 0;
    return closingPrefixAfter;
    function closingPrefixAfter(code) {
      sectionIndentSize = linePrefixSize(self.events);
      effects2.exit("componentContainerSection");
      effects2.enter("componentContainerSectionSequence");
      return closingSectionSequence(code);
    }
    function closingSectionSequence(code) {
      if (code === sectionSeparatorCode) {
        effects2.consume(code);
        size++;
        return closingSectionSequence;
      }
      if (size !== sectionSeparatorLength) {
        return nok2(code);
      }
      if (sectionIndentSize !== initialPrefix) {
        return nok2(code);
      }
      if (!asciiAlpha(code)) {
        return nok2(code);
      }
      effects2.exit("componentContainerSectionSequence");
      return factorySpace(effects2, ok2, "whitespace")(code);
    }
  }
  function sectionOpen(code) {
    effects.enter("componentContainerSection");
    if (markdownLineEnding(code)) {
      return factorySpace(effects, lineStart, "whitespace")(code);
    }
    effects.enter("componentContainerSectionTitle");
    return sectionTitle;
  }
  function sectionTitle(code) {
    if (markdownLineEnding(code)) {
      effects.exit("componentContainerSectionTitle");
      return factorySpace(effects, lineStart, "linePrefix", 4)(code);
    }
    effects.consume(code);
    return sectionTitle;
  }
  function sequenceOpen(code) {
    if (code === Codes.colon) {
      effects.consume(code);
      sizeOpen++;
      return sequenceOpen;
    }
    if (sizeOpen < ContainerSequenceSize) {
      return nok(code);
    }
    effects.exit("componentContainerSequence");
    return createName.call(self, effects, afterName, nok, "componentContainerName")(code);
  }
  function afterName(code) {
    return code === Codes.openingSquareBracket ? effects.attempt(label, afterLabel, afterLabel)(code) : afterLabel(code);
  }
  function afterLabel(code) {
    return code === Codes.openingCurlyBracket ? effects.attempt(attributes, afterAttributes, afterAttributes)(code) : afterAttributes(code);
  }
  function afterAttributes(code) {
    return factorySpace(effects, openAfter, "whitespace")(code);
  }
  function openAfter(code) {
    effects.exit("componentContainerFence");
    if (code === null) {
      effects.exit("componentContainer");
      return ok(code);
    }
    if (markdownLineEnding(code)) {
      effects.enter("lineEnding");
      effects.consume(code);
      effects.exit("lineEnding");
      return self.interrupt ? ok : contentStart;
    }
    return nok(code);
  }
  function contentStart(code) {
    if (code === null) {
      effects.exit("componentContainer");
      return ok(code);
    }
    effects.enter("componentContainerContent");
    if (!containerSequenceSize.length && !data.isClosed() && (code === Codes.dash || markdownSpace(code))) {
      let _chunkStart = function(code2) {
        data.close();
        effects.enter("componentContainerSection");
        return lineStart(code2);
      };
      return effects.attempt(data.tokenize, data.sectionOpen, _chunkStart);
    } else {
      data.close();
    }
    effects.enter("componentContainerSection");
    return lineStart(code);
  }
  function lineStartAfterPrefix(code) {
    if (code === null) {
      return after(code);
    }
    if (!containerSequenceSize.length && (code === sectionSeparatorCode || code === Codes.space)) {
      return effects.attempt({ tokenize: tokenizeSectionClosing, partial: true }, sectionOpen, chunkStart);
    }
    if (!containerSequenceSize.length && !data.isClosed() && (code === Codes.dash || code === Codes.space)) {
      return effects.attempt(data.tokenize, data.sectionOpen, chunkStart);
    }
    const attempt = effects.attempt({ tokenize: tokenizeClosingFence, partial: true }, after, chunkStart);
    if (code === Codes.colon) {
      return effects.check({ tokenize: detectContainer, partial: true }, chunkStart, attempt)(code);
    }
    return attempt;
  }
  function lineStart(code) {
    if (code === null) {
      return after(code);
    }
    return initialPrefix ? factorySpace(effects, lineStartAfterPrefix, "linePrefix", initialPrefix + 1) : lineStartAfterPrefix;
  }
  function chunkStart(code) {
    if (code === null) {
      return after(code);
    }
    const token = effects.enter("chunkDocument", {
      contentType: "document",
      previous
    });
    if (previous) {
      previous.next = token;
    }
    previous = token;
    return contentContinue(code);
  }
  function contentContinue(code) {
    if (code === null) {
      effects.exit("chunkDocument");
      return after(code);
    }
    if (markdownLineEnding(code)) {
      effects.consume(code);
      effects.exit("chunkDocument");
      return lineStart;
    }
    effects.consume(code);
    return contentContinue;
  }
  function after(code) {
    effects.exit("componentContainerSection");
    effects.exit("componentContainerContent");
    effects.exit("componentContainer");
    return ok(code);
  }
  function tokenizeClosingFence(effects2, ok2, nok2) {
    let size = 0;
    return factorySpace(effects2, closingPrefixAfter, "linePrefix", 4);
    function closingPrefixAfter(code) {
      effects2.enter("componentContainerFence");
      effects2.enter("componentContainerSequence");
      return closingSequence(code);
    }
    function closingSequence(code) {
      if (code === Codes.colon) {
        effects2.consume(code);
        size++;
        return closingSequence;
      }
      if (containerSequenceSize.length) {
        if (size === containerSequenceSize[containerSequenceSize.length - 1]) {
          containerSequenceSize.pop();
        }
        return nok2(code);
      }
      if (size !== sizeOpen) {
        return nok2(code);
      }
      effects2.exit("componentContainerSequence");
      return factorySpace(effects2, closingSequenceEnd, "whitespace")(code);
    }
    function closingSequenceEnd(code) {
      if (code === null || markdownLineEnding(code)) {
        effects2.exit("componentContainerFence");
        return ok2(code);
      }
      return nok2(code);
    }
  }
  function detectContainer(effects2, ok2, nok2) {
    let size = 0;
    return openingSequence;
    function openingSequence(code) {
      if (code === Codes.colon) {
        effects2.consume(code);
        size++;
        return openingSequence;
      }
      if (size < ContainerSequenceSize) {
        return nok2(code);
      }
      return openingSequenceEnd;
    }
    function openingSequenceEnd(code) {
      if (code === null || markdownLineEnding(code)) {
        return nok2(code);
      }
      containerSequenceSize.push(size);
      return ok2(code);
    }
  }
}
function tokenizeData(effects, ok) {
  const initialPrefix = linePrefixSize(this.events);
  let sectionState = 0;
  const data = {
    state: () => sectionState,
    close: () => {
      sectionState = 2;
    },
    isClosed: () => sectionState === 2,
    tokenize: { tokenize: tokenizeDataSection, partial: true },
    sectionOpen
  };
  return data;
  function tokenizeDataSection(effects2, ok2, nok) {
    const self = this;
    let size = 0;
    let sectionIndentSize = 0;
    return closingPrefixAfter;
    function closingPrefixAfter(code) {
      if (data.isClosed()) {
        return nok(code);
      }
      if (markdownSpace(code)) {
        effects2.consume(code);
        sectionIndentSize += 1;
        return closingPrefixAfter;
      }
      if (sectionIndentSize === 0) {
        sectionIndentSize = linePrefixSize(self.events);
      }
      if (sectionState === 1) {
        effects2.exit("componentContainerDataSection");
      }
      effects2.enter("componentContainerSectionSequence");
      return closingSectionSequence(code);
    }
    function closingSectionSequence(code) {
      if (code === Codes.dash || markdownSpace(code)) {
        effects2.consume(code);
        size++;
        return closingSectionSequence;
      }
      if (size < SectionSequenceSize) {
        return nok(code);
      }
      if (sectionIndentSize !== initialPrefix) {
        return nok(code);
      }
      if (!markdownLineEnding(code)) {
        return nok(code);
      }
      effects2.exit("componentContainerSectionSequence");
      return factorySpace(effects2, ok2, "whitespace")(code);
    }
  }
  function sectionOpen(code) {
    if (sectionState === 0) {
      effects.enter("componentContainerDataSection");
      sectionState = 1;
    } else {
      sectionState = 2;
      effects.enter("componentContainerSection");
    }
    return factorySpace(effects, ok, "whitespace")(code);
  }
}
function tokenizeLabel(effects, ok, nok) {
  return createLabel(effects, ok, nok, "componentContainerLabel", "componentContainerLabelMarker", "componentContainerLabelString", true);
}
function tokenizeAttributes(effects, ok, nok) {
  return createAttributes(effects, ok, nok, "componentContainerAttributes", "componentContainerAttributesMarker", "componentContainerAttribute", "componentContainerAttributeId", "componentContainerAttributeClass", "componentContainerAttributeName", "componentContainerAttributeInitializerMarker", "componentContainerAttributeValueLiteral", "componentContainerAttributeValue", "componentContainerAttributeValueMarker", "componentContainerAttributeValueData", true);
}
export default {
  tokenize,
  concrete: true
};
