import { apply, ensureArray, omit, pick, sortByKey } from "./utils.mjs";
import { createMatch } from "./index.mjs";
export function createPipelineFetcher(getContentsList) {
  const match = createMatch();
  const surround = (data, { query, before, after }) => {
    const matchQuery = typeof query === "string" ? { slug: query } : query;
    const index = data.findIndex((item) => match(item, matchQuery));
    before = before || 1;
    after = after || 1;
    const slice = new Array(before + after).fill(null, 0);
    return index === -1 ? slice : slice.map((_, i) => data[index - before + i + Number(i >= before)] || null);
  };
  const pipelines = [
    (data, params) => params.slug ? data.filter((item) => String(item.slug).startsWith(params.slug)) : data,
    (data, params) => data.filter((item) => ensureArray(params.where).every((matchQuery) => match(item, matchQuery))),
    (data, params) => ensureArray(params.sortBy).forEach(([key, direction]) => sortByKey(data, key, direction)),
    (data, params) => params.surround ? surround(data, params.surround) : data,
    (data, params) => params.skip ? data.slice(params.skip) : data,
    (data, params) => params.limit ? data.slice(0, params.limit) : data,
    (data, params) => apply(omit(params.without))(data),
    (data, params) => apply(pick(params.only))(data),
    (data, params) => params.first ? data[0] : data
  ];
  return async (params) => {
    const data = await getContentsList();
    if (!params.sortBy || !params.sortBy.length) {
      params.sortBy = [
        ["path", "asc"]
      ];
    }
    return pipelines.reduce(($data, pipe) => pipe($data, params) || $data, data);
  };
}
