export const get = (obj, path) => path.split(".").reduce((acc, part) => acc && acc[part], obj);
const _pick = (obj, condition) => Object.keys(obj).filter(condition).reduce((newObj, key) => Object.assign(newObj, { [key]: obj[key] }), {});
export const pick = (keys) => (obj) => keys && keys.length ? _pick(obj, (key) => keys.includes(key)) : obj;
export const omit = (keys) => (obj) => keys && keys.length ? _pick(obj, (key) => !keys.includes(key)) : obj;
export const apply = (fn) => (data) => Array.isArray(data) ? data.map((item) => fn(item)) : fn(data);
export const sortComparable = (key, direction) => (a, b) => {
  const values = [get(a, key), get(b, key)];
  if (direction === "desc" || direction === true) {
    values.reverse();
  }
  if (values[0] === void 0) {
    return 1;
  }
  if (values[1] === void 0) {
    return -1;
  }
  if (values[0] > values[1]) {
    return 1;
  }
  if (values[0] < values[1]) {
    return -1;
  }
  return 0;
};
export const sortByKey = (data, key, direction) => data.sort(sortComparable(key, direction));
export const assertArray = (value, message = "Expected an array") => {
  if (!Array.isArray(value)) {
    throw new TypeError(message);
  }
};
export const ensureArray = (value) => Array.isArray(value) ? value : [value];
