import { ensureArray } from "./match/utils.mjs";
export const createQuery = (fetcher, queryParams) => {
  const params = {
    sortBy: [],
    ...queryParams,
    where: queryParams?.where ? ensureArray(queryParams.where) : [],
    only: queryParams?.only ? ensureArray(queryParams.only) : [],
    without: queryParams?.without ? ensureArray(queryParams.without) : []
  };
  const $set = (key, fn = (v) => v) => {
    return (...values) => {
      params[key] = fn(...values);
      return query;
    };
  };
  const query = {
    params: () => Object.freeze(params),
    only: $set("only", ensureArray),
    without: $set("without", ensureArray),
    where: $set("where", (q) => [...params.where, q]),
    sortBy: $set("sortBy", (field, direction) => [...params.sortBy, [field, direction]]),
    limit: $set("limit", (v) => parseInt(String(v), 10)),
    skip: $set("skip", (v) => parseInt(String(v), 10)),
    findOne: () => fetcher({ ...params, first: true }),
    find: () => fetcher(params),
    findSurround: (query2, options) => fetcher({ ...params, surround: { query: query2, ...options } }),
    locale: (locale) => query.where({ locale })
  };
  return query;
};
