import { createError, defineLazyEventHandler } from "h3";
import { getHighlighter, BUNDLED_LANGUAGES, BUNDLED_THEMES } from "shiki-es";
import { contentApiParams } from "../utils.mjs";
import { useRuntimeConfig } from "#nitro";
const resolveLang = (lang) => BUNDLED_LANGUAGES.find((l) => l.id === lang || l.aliases?.includes(lang))?.id;
const resolveTheme = (theme) => BUNDLED_THEMES.find((t) => t === theme);
const resolveBody = (body) => {
  if (typeof body.code !== "string") {
    throw createError({ statusMessage: "Bad Request", statusCode: 400, message: "Missing code key." });
  }
  return {
    code: body.code.replace(/\n+$/, ""),
    lang: resolveLang(body.lang),
    theme: resolveTheme(body.theme)
  };
};
export default defineLazyEventHandler(async () => {
  const { theme, preload } = useRuntimeConfig().content.highlight;
  const highlighter = await getHighlighter({
    theme: theme || "dark-plus",
    langs: preload || ["json", "js", "ts", "html", "css"]
  });
  return async (event) => {
    const params = contentApiParams(event);
    const { code, lang, theme: theme2 } = resolveBody(params);
    if (!lang) {
      return [[{ content: code }]];
    }
    if (!highlighter.getLoadedLanguages().includes(lang)) {
      await highlighter.loadLanguage(lang);
    }
    if (theme2 && !highlighter.getLoadedThemes().includes(theme2)) {
      await highlighter.loadTheme(theme2);
    }
    const highlightedCode = highlighter.codeToThemedTokens(code, lang, theme2);
    for (const line of highlightedCode) {
      for (const token of line) {
        delete token.fontStyle;
        delete token.explanation;
      }
    }
    return highlightedCode;
  };
});
