import { generateTitle } from "./transformers/path-meta.mjs";
export function createNav(contents) {
  const nav = contents.sort((a, b) => a.slug.localeCompare(b.slug)).reduce((nav2, content) => {
    const parts = content.slug.substring(1).split("/");
    const idParts = content.id.split(":").slice(1);
    const isIndex = !!idParts[idParts.length - 1].match(/([1-9][0-9]*\.)?index.md/g);
    const getNavItem = (content2) => {
      return {
        id: content2.id,
        title: content2.title,
        slug: content2.slug,
        draft: content2.draft,
        partial: content2.partial,
        path: content2.path,
        children: []
      };
    };
    const navItem = getNavItem(content);
    if (isIndex) {
      const indexItem = getNavItem(content);
      navItem.children.push(indexItem);
      if (content.parent) {
        if (content.parent.title && typeof content.parent.title === "string") {
          navItem.title = content.parent.title;
        }
      }
      delete navItem.id;
      delete navItem.draft;
      delete navItem.partial;
    }
    if (parts.length === 1) {
      nav2.push(navItem);
      return nav2;
    }
    const siblings = parts.slice(0, -1).reduce((nodes, part, i) => {
      const currentPathSlug = "/" + parts.slice(0, i + 1).join("/");
      let parent = nodes.find((n) => n.slug === currentPathSlug);
      if (!parent) {
        parent = {
          slug: currentPathSlug,
          title: generateTitle(part),
          path: idParts.slice(0, i + 1).join(":"),
          children: []
        };
        nodes.push(parent);
      }
      return parent.children;
    }, nav2);
    siblings.push(navItem);
    return nav2;
  }, []);
  return sortAndClear(nav);
}
function sortAndClear(nav) {
  const sorted = nav.sort((a, b) => a.path.localeCompare(b.path));
  for (const item of sorted) {
    if (item.children.length) {
      sortAndClear(item.children);
    } else {
      delete item.children;
    }
    delete item.path;
  }
  return nav;
}
