import { prefixStorage } from "unstorage";
import { isDevelopment } from "std-env";
import { hash as ohash } from "ohash";
import { createQuery } from "../query/query.mjs";
import { createPipelineFetcher } from "../query/match/pipeline.mjs";
import { parse, transform } from "./transformers/index.mjs";
import { useRuntimeConfig, useStorage } from "#imports";
export const contentStorage = isDevelopment ? prefixStorage(useStorage(), "content:source") : prefixStorage(useStorage(), "assets:content:source");
export const cacheStorage = prefixStorage(useStorage(), "cache:content");
export const contentIgnores = useRuntimeConfig().content.ignores.map((p) => typeof p === "string" ? new RegExp(`^${p}`) : p);
const contentIgnorePredicate = (key) => !contentIgnores.some((prefix) => key.split(":").some((k) => prefix.test(k)));
export const getContentsIds = async (prefix) => {
  const keys = await contentStorage.getKeys(prefix);
  return keys.filter(contentIgnorePredicate);
};
export const getContentsList = async (prefix) => {
  const keys = await getContentsIds(prefix);
  const contents = await Promise.all(keys.map((key) => getContent(key)));
  return contents;
};
export const getContent = async (id) => {
  if (!contentIgnorePredicate(id)) {
    return { id, body: null };
  }
  const hash = ohash(await contentStorage.getMeta(id));
  const cached = await cacheStorage.getItem(`parsed:${id}`);
  if (cached?.hash === hash) {
    return cached.parsed;
  }
  const body = await contentStorage.getItem(id);
  if (body === null) {
    return { id, body: null };
  }
  const meta = await contentStorage.getMeta(id);
  const parsedContent = await parse(id, body).then(transform);
  const parsed = {
    ...meta,
    ...parsedContent
  };
  await cacheStorage.setItem(`parsed:${id}`, { parsed, hash });
  return parsed;
};
export const queryContent = (body, params) => {
  if (typeof body === "string") {
    body = {
      slug: body,
      ...params
    };
  }
  const pipelineFetcher = createPipelineFetcher(getContentsList);
  return createQuery(pipelineFetcher, body);
};
